require 'spec_helper'
require 'support/shared_missing_admin'

describe DashOverlord::UseCases::V1::Admins::ShowCurrentAdmin::Base do
  def make_the_call(params = {})
    @context = described_class.perform \
      session: admin_session

    raise_context_unknown_error @context

    @response = app_response @context
  end

  def admin_attributes(*args)
    attributes = attributes_for(*args).slice \
      :id,
      :email

    attributes
  end

  include_examples 'authenticated admin use_case'

  context 'when current_admin exists' do
    before { @current_admin = create('v1/admin', :streetbees) }

    context 'when requesting an existing user' do
      before do
        @user = create('v1/admin')

        make_the_call
      end

      it 'response.http_status should return 200' do
        expect(@response.http_status).to be 200
      end

      it 'response.data should return Admin details' do
        attributes = admin_attributes('v1/admin', :streetbees)

        attributes[:id] = @current_admin.id

        expect(@response.data).to match a_hash_including({
          'code' => 200,
          'status' => 'ok',
          'data' => a_hash_including(attributes)
        })
      end
    end
  end
end
