require 'spec_helper'
require 'support/shared_missing_admin'

describe DashOverlord::UseCases::V1::ChartConfigs::Create::Base do
  def make_the_call(params = {})
    @context = described_class.perform \
      chart_config_attributes: (params || {}), session: admin_session

    raise_context_unknown_error @context

    @response = app_response @context
  end

  def chart_config_attributes(*args)
    attributes = attributes_for(*args).slice \
      :id,
      :source,
      :function,
      :project_id

    attributes[:source] = attributes[:source].to_s

    attributes
  end

  include_examples 'authenticated admin use_case'

  context 'when current_admin exists' do
    before { @current_admin = create('v1/admin', :streetbees) }

    context 'when sending valid params' do
      before do
        make_the_call chart_config_attributes('v1/chart_config', :pie_chart)
      end

      it 'response.http_status should return 201' do
        expect(@response.http_status).to be 201
      end

      it 'response.data should return ChartConfig details' do
        chart_config = DashOverlord::Models::V1::ChartConfig.last

        attributes = chart_config_attributes('v1/chart_config', :pie_chart)

        expect(@response.data).to match a_hash_including({
          'code' => 201,
          'status' => 'created',
          'data' => a_hash_including({ id: chart_config.id }.merge(attributes))
        })
      end
    end

    context 'when sending invalid params' do
      before do
        make_the_call
      end

      it 'response.http_status should return 422' do
        expect(@response.http_status).to be 422
      end

      it 'response.data should return ChartConfig details' do
        expect(@response.data).to match a_hash_including({
          'code' => 422,
          'status' => 'unprocessable_entity',
          'data' => a_hash_including({
            'errors' => a_hash_including(
              'source' => [I18n.t('errors.messages.blank')],
              'function' => [I18n.t('errors.messages.invalid')],
              'project_id' => [I18n.t('errors.messages.blank')]
            )
          })
        })
      end
    end
  end
end
