require 'spec_helper'
require 'support/shared_missing_admin'

describe DashOverlord::UseCases::V1::ChartConfigs::Delete::Base do
  def make_the_call(chart_config_id, params = {})
    @context = described_class.perform \
      session: admin_session,
      chart_config_id: chart_config_id

    raise_context_unknown_error @context

    @response = app_response @context
  end

  include_examples 'authenticated admin use_case'

  context 'when current_admin exists' do
    before { @current_admin = create('v1/admin', :streetbees) }

    context 'when no record exists' do
      before do
        create('v1/chart_config')
        @chart_config = create('v1/chart_config', :pie_chart)

        make_the_call @chart_config.id
      end

      it 'response.http_status should return 202' do
        expect(@response.http_status).to be 202
      end

      it 'No extra ChartConfig should be deleted' do
        expect(DashOverlord::Models::V1::ChartConfig.count).to be 1
      end

      it 'response.data should not return ChartConfig details' do
        expect(@response.data).to match a_hash_including({
          'code' => 202,
          'status' => 'accepted',
          'data' => nil
        })
      end
    end

    context 'when a record exists' do
      before do
        create('v1/chart_config')
        @chart_config = create('v1/chart_config', :pie_chart)

        make_the_call @chart_config.id
      end

      it 'response.http_status should return 202' do
        expect(@response.http_status).to be 202
      end

      it 'No extra ChartConfig should be deleted' do
        expect(DashOverlord::Models::V1::ChartConfig.count).to be 1
      end

      it 'response.data should not return ChartConfig details' do
        expect(@response.data).to match a_hash_including({
          'code' => 202,
          'status' => 'accepted',
          'data' => nil
        })
      end
    end
  end
end
