require 'spec_helper'
require 'support/shared_missing_admin'

describe DashOverlord::UseCases::V1::ChartConfigs::Show::Base do
  def make_the_call(id, params = {})
    @context = described_class.perform \
      chart_config_id: id, session: admin_session

    raise_context_unknown_error @context

    @response = app_response @context
  end

  def chart_config_attributes(*args)
    attributes = attributes_for(*args).slice \
      :id,
      :source,
      :function,
      :project_id

    attributes[:source] = attributes[:source].to_s

    attributes
  end

  include_examples 'authenticated admin use_case'

  context 'when current_admin exists' do
    before { @current_admin = create('v1/admin', :streetbees) }

    context 'when requesting an non existing ChartConfig' do
      before do
        make_the_call -1
      end

      it 'response.http_status should return 404' do
        expect(@response.http_status).to be 404
      end

      it 'response.data should return a 404 message' do
        expect(@response.data).to match a_hash_including({
          'code' => 404,
          'message' => I18n.t('v1.chart_configs.not_found'),
          'status' => 'not_found'
        })
      end
    end

    context 'when requesting an existing ChartConfig' do
      before do
        @chart_config = create('v1/chart_config', :pie_chart)

        make_the_call @chart_config.id
      end

      it 'response.http_status should return 200' do
        expect(@response.http_status).to be 200
      end

      it 'response.data should return ChartConfig details' do
        attributes = chart_config_attributes('v1/chart_config', :pie_chart)

        attributes[:id] = @chart_config.id

        expect(@response.data).to match a_hash_including({
          'code' => 200,
          'status' => 'ok',
          'data' => a_hash_including(attributes)
        })
      end
    end
  end
end
