require 'spec_helper'
require 'support/shared_missing_admin'

describe DashOverlord::UseCases::V1::Clients::Create::Base do
  def make_the_call(params = {})
    @context = described_class.perform \
      client_attributes: (params || {}), session: admin_session

    raise_context_unknown_error @context

    @response = app_response @context
  end

  def client_attributes(*args)
    attributes = attributes_for(*args).slice \
      :id,
      :uid,
      :name

    attributes
  end

  include_examples 'authenticated admin use_case'

  context 'when current_admin exists' do
    before { @current_admin = create('v1/admin', :streetbees) }

    context 'when sending valid params' do
      before do
        make_the_call client_attributes('v1/client', :unilever)
      end

      it 'response.http_status should return 201' do
        expect(@response.http_status).to be 201
      end

      it 'response.data should return Client details' do
        client = DashOverlord::Models::V1::Client.last

        attributes = client_attributes('v1/client', :unilever)
        expect(@response.data).to match a_hash_including({
          'code' => 201,
          'status' => 'created',
          'data' => { id: client.id }.merge(attributes)
        })
      end
    end

    context 'when sending invalid params' do
      before do
        make_the_call
      end

      it 'response.http_status should return 422' do
        expect(@response.http_status).to be 422
      end

      it 'response.data should return Client details' do
        expect(@response.data).to match a_hash_including({
          'code' => 422,
          'status' => 'unprocessable_entity',
          'data' => a_hash_including({
            'errors' => a_hash_including(
              'name' => [I18n.t('errors.messages.blank')]
            )
          })
        })
      end
    end

    context 'when trying to overwrite the uid' do
      before do
        make_the_call name: 'Unilever', uid: 'vodafone'
      end

      it 'existing Client must belong to current_admin' do
        client = DashOverlord::Models::V1::Client.last

        attributes = client_attributes('v1/client', :unilever)

        expect(client.uid).to eq attributes[:uid]
      end
    end
  end
end
