require 'spec_helper'
require 'support/shared_missing_admin'

describe DashOverlord::UseCases::V1::Users::Create::Base do
  def make_the_call(params = {})
    @context = described_class.perform \
      user_attributes: (params || {}), session: admin_session

    raise_context_unknown_error @context

    @response = app_response @context
  end

  def user_attributes(*args)
    attributes = attributes_for(*args).slice \
      :id,
      :email

    attributes
  end

  include_examples 'authenticated admin use_case'

  context 'when current_admin exists' do
    before { @current_admin = create('v1/admin', :streetbees) }

    context 'when sending valid params' do
      before do
        make_the_call email: 'admin@unilever.com', password: '123456'
      end

      it 'response.http_status should return 201' do
        expect(@response.http_status).to be 201
      end

      it 'response.data should return User details' do
        user = DashOverlord::Models::V1::User.last

        attributes = user_attributes('v1/user', :unilever)
        expect(@response.data).to match a_hash_including({
          'code' => 201,
          'status' => 'created',
          'data' => { id: user.id }.merge(attributes)
        })
      end

      it 'existing User password must be corrrect' do
        user = DashOverlord::Models::V1::User.last

        expect(user.secured_password).to eq '123456'
      end
    end

    context 'when sending invalid params' do
      before do
        make_the_call
      end

      it 'response.http_status should return 422' do
        expect(@response.http_status).to be 422
      end

      it 'response.data should return User details' do
        expect(@response.data).to match a_hash_including({
          'code' => 422,
          'status' => 'unprocessable_entity',
          'data' => a_hash_including({
            'errors' => a_hash_including(
              'email' => [I18n.t('errors.messages.blank')],
              'password' => [I18n.t('errors.messages.blank')]
            )
          })
        })
      end
    end

    context 'when trying to overwrite the admin field' do
      before do
        make_the_call email: 'admin@unilever.com', password: '123456', admin: true
      end

      it 'existing User must belong to current_admin' do
        user = DashOverlord::Models::V1::User.last

        expect(user.admin).to be_falsey
      end
    end
  end
end
