require 'spec_helper'
require 'support/shared_missing_admin'

describe DashOverlord::UseCases::V1::Users::Index::Base do
  def make_the_call(params = nil)
    @context = described_class.perform \
      (params || {}).merge({ session: admin_session })

    raise_context_unknown_error @context

    @response = app_response @context
  end

  include_examples 'authenticated admin use_case'

  context 'when current_admin exists' do
    before { @current_admin = create('v1/admin', :streetbees) }

    context 'when no users are available' do
      before { make_the_call }

      it 'response.status should return 200' do
        expect(@response.http_status).to be 200
      end

      it 'should return pagination meta_data' do
        expect(@response.data).to match a_hash_including({
          'meta' => a_hash_including({
            'page' => 1,
            'per_page'=> 10,
            'total_pages' => 0,
            'total_results' => 0
          }),
          'data' => []
        })
      end
    end

    context 'when no filter params are used' do
      before do
        20.times { |index| create('v1/user', email: "#{index}@streetbees.com") }

        make_the_call
      end

      it 'response.status should return 200' do
        expect(@response.http_status).to be 200
      end

      it 'should return pagination meta_data' do
        expect(@response.data).to match a_hash_including({
          'meta' => a_hash_including({
            'page' => 1,
            'per_page'=> 10,
            'total_pages' => 2,
            'total_results' => 20
          })
        })
      end

      it 'should return 10 results' do
        expect(@response.data['data'].length).to be 10
      end
    end

    context 'when filtering by page = 0' do
      before do
        20.times { |index| create('v1/user', email: "#{index}@streetbees.com") }

        make_the_call page: 0
      end

      it 'response.status should return 200' do
        expect(@response.http_status).to be 200
      end

      it 'should return pagination meta_data' do
        expect(@response.data).to match a_hash_including({
          'meta' => a_hash_including({
            'page' => 0,
            'per_page'=> 0,
            'total_pages' => 0,
            'total_results' => 20
          })
        })
      end

      it 'should return 20 results' do
        expect(@response.data['data'].length).to be 20
      end
    end

    context 'when filtering by something' do
      before do
        5.times { |index| create('v1/user', email: "#{index}@unilever.com") }
        15.times { |index| create('v1/user', email: "#{index}@streetbees.com") }

        make_the_call filter_query: { email_cont: 'uni' }
      end

      it 'response.status should return 200' do
        expect(@response.http_status).to be 200
      end

      it 'should return pagination meta_data' do
        expect(@response.data).to match a_hash_including({
          'meta' => a_hash_including({
            'page' => 1,
            'per_page'=> 10,
            'total_pages' => 1,
            'total_results' => 5
          })
        })
      end

      it 'should return 5 results' do
        expect(@response.data['data'].length).to be 5
      end
    end
  end
end
