require 'spec_helper'

describe DashOverlord::UseCases::V1::Users::LogIn::Base do
  def make_the_call(params = {})
    @context = described_class.perform \
      user_attributes: params

    raise_context_unknown_error @context

    @response = app_response @context
  end

  context 'when sending valid params' do
    before do
      @user = create('v1/user', :unilever)

      make_the_call email: 'admin@unilever.com', password: 'password'
    end

    it 'response.http_status should return 200' do
      expect(@response.http_status).to be 200
    end

    it 'response.data should return User details' do
      expect(@response.data).to match a_hash_including({
        'code' => 200,
        'status' => 'ok',
        'data' => a_hash_including({ id: @user.id })
      })
    end
  end

  context 'when not sending any params' do
    before do
      make_the_call
    end

    it 'response.http_status should return 422' do
      expect(@response.http_status).to be 422
    end

    it 'response.data should return User details' do
      expect(@response.data).to match a_hash_including({
        'code' => 422,
        'status' => 'unprocessable_entity',
        'data' => a_hash_including({
          'errors' => a_hash_including(
            'email' => [I18n.t('errors.messages.blank')],
            'password' => [I18n.t('errors.messages.blank')]
          )
        })
      })
    end
  end

  context 'when sending invalid password' do
    before do
      @user = create('v1/user', :unilever)

      make_the_call email: 'admin@unilever.com', password: 'fake'
    end

    it 'response.http_status should return 422' do
      expect(@response.http_status).to be 422
    end

    it 'response.data should return User details' do
      expect(@response.data).to match a_hash_including({
        'code' => 422,
        'status' => 'unprocessable_entity',
        'data' => a_hash_including({
          'errors' => a_hash_including(
            'email' => [I18n.t('v1.users.bad_credentials')],
            'password' => [I18n.t('v1.users.bad_credentials')]
          )
        })
      })
    end
  end

  context 'when sending invalid email' do
    before do
      @user = create('v1/user', :unilever)

      make_the_call email: 'fake', password: 'password'
    end

    it 'response.http_status should return 422' do
      expect(@response.http_status).to be 422
    end

    it 'response.data should return User details' do
      expect(@response.data).to match a_hash_including({
        'code' => 422,
        'status' => 'unprocessable_entity',
        'data' => a_hash_including({
          'errors' => a_hash_including(
            'email' => [I18n.t('v1.users.bad_credentials')],
            'password' => [I18n.t('v1.users.bad_credentials')]
          )
        })
      })
    end
  end
end
