require 'spec_helper'
require 'support/shared_missing_user'

describe DashOverlord::UseCases::V1::Users::ShowCurrentUser::Base do
  def make_the_call(params = {})
    @context = described_class.perform \
      session: user_session

    raise_context_unknown_error @context

    @response = app_response @context
  end

  def user_attributes(*args)
    attributes = attributes_for(*args).slice \
      :id,
      :email

    attributes
  end

  include_examples 'authenticated user use_case'

  context 'when current_user exists' do
    before { @current_user = create('v1/user', :unilever) }

    context 'when requesting an existing user' do
      before do
        @user = create('v1/user')

        make_the_call
      end

      it 'response.http_status should return 200' do
        expect(@response.http_status).to be 200
      end

      it 'response.data should return User details' do
        attributes = user_attributes('v1/user', :unilever)

        attributes[:id] = @current_user.id

        expect(@response.data).to match a_hash_including({
          'code' => 200,
          'status' => 'ok',
          'data' => a_hash_including(attributes)
        })
      end
    end
  end
end
