require 'spec_helper'
require 'support/shared_missing_admin'

describe DashOverlord::UseCases::V1::Users::Show::Base do
  def make_the_call(id, params = {})
    @context = described_class.perform \
      user_id: id, session: admin_session

    raise_context_unknown_error @context

    @response = app_response @context
  end

  def user_attributes(*args)
    attributes = attributes_for(*args).slice \
      :id,
      :email

    attributes
  end

  include_examples 'authenticated admin use_case'

  context 'when current_admin exists' do
    before { @current_admin = create('v1/admin', :streetbees) }

    context 'when requesting an non existing user' do
      before do
        make_the_call -1
      end

      it 'response.http_status should return 404' do
        expect(@response.http_status).to be 404
      end

      it 'response.data should return a 404 message' do
        expect(@response.data).to match a_hash_including({
          'code' => 404,
          'message' => I18n.t('v1.users.not_found'),
          'status' => 'not_found'
        })
      end
    end

    context 'when requesting an existing user' do
      before do
        @user = create('v1/user', :unilever)

        make_the_call @user.id
      end

      it 'response.http_status should return 200' do
        expect(@response.http_status).to be 200
      end

      it 'response.data should return User details' do
        attributes = user_attributes('v1/user', :unilever)

        attributes[:id] = @user.id

        expect(@response.data).to match a_hash_including({
          'code' => 200,
          'status' => 'ok',
          'data' => a_hash_including(attributes)
        })
      end
    end
  end
end
