require 'spec_helper'

describe DashOverlord::UseCases::V1::Videos::Index::ElasticSearch::Base,
         elasticsearch: true do
  def bee_video_client(segment_uid)
    DashOverlord::Models::V1::ElasticSearch::BeeVideo.by_dashboard(segment_uid)
  end

  def make_the_call(params = nil)
    bee_video_client(params[:segment_uid]).refresh_index

    @context = described_class.perform \
      (params || {}).merge({ session: user_session })

    raise_context_unknown_error @context
  end

  def create_answer(segment_uid, attributes = {})
    bee_video_client(segment_uid).create attributes
  end

  context 'when no bee_videos are available' do
    before(:all) do
      bee_video_client('hhc').delete_all

      make_the_call segment_uid: 'hhc'
    end

    it '@context.status should be ok' do
      expect(@context.status).to eq 'ok'
    end

    it 'should return pagination meta_data' do
      expect(@context.meta.page).to be 1
      expect(@context.meta.per_page).to be 10
      expect(@context.meta.total_pages).to be 0
      expect(@context.meta.total_results).to be 0
    end

    it 'should return 0 results' do
      expect(@context.videos.length).to be 0
    end
  end

  context 'when no filter params are used' do
    before(:all) do
      bee_video_client('hhc').delete_all

      20.times { create_answer('hhc', title: 'Test') }

      make_the_call segment_uid: 'hhc'
    end

    it '@context.status should be ok' do
      expect(@context.status).to eq 'ok'
    end

    it 'should return pagination meta_data' do
      expect(@context.meta.page).to be 1
      expect(@context.meta.per_page).to be 10
      expect(@context.meta.total_pages).to be 2
      expect(@context.meta.total_results).to be 20
    end

    it 'should return 10 results' do
      expect(@context.videos.length).to be 10
    end
  end

  context 'when filtering by page = 3' do
    before(:all) do
      bee_video_client('hhc').delete_all

      35.times { create_answer('hhc', title: 'Test') }

      make_the_call segment_uid: 'hhc', page: 3, per_page: 15
    end

    it '@context.status should be ok' do
      expect(@context.status).to eq 'ok'
    end

    it 'should return pagination meta_data' do
      expect(@context.meta.page).to be 3
      expect(@context.meta.per_page).to be 15
      expect(@context.meta.total_pages).to be 3
      expect(@context.meta.total_results).to be 35
    end

    it 'should return 5 results' do
      expect(@context.videos.length).to be 5
    end
  end

  context 'when filtering by something' do
    before(:all) do
      bee_video_client('hhc').delete_all

      15.times { create_answer('laundry', user_id: 1, gender: 'Female') }
      10.times { create_answer('hhc', user_id: 1, gender: 'Male') }
      5.times { create_answer('hhc', user_id: 1, gender: 'Female') }
      20.times { create_answer('hhc', user_id: 2, gender: 'Female') }

      make_the_call \
        segment_uid: 'hhc',
        filter_query: {
          user_id_eq: 1,
          gender_eq: 'Female'
        }
    end

    it '@context.status should be ok' do
      expect(@context.status).to eq 'ok'
    end

    it 'should return pagination meta_data' do
      expect(@context.meta.page).to be 1
      expect(@context.meta.per_page).to be 10
      expect(@context.meta.total_pages).to be 1
      expect(@context.meta.total_results).to be 5
    end

    it 'should return 5 results' do
      expect(@context.videos.length).to be 5
    end
  end
end
