require 'spec_helper'

describe DashOverlord::UseCases::V1::Videos::Index::Postgres::Base do
  def make_the_call(params = nil)
    @context = described_class.perform \
      (params || {}).merge({ session: user_session })

    raise_context_unknown_error @context
  end

  context 'when no bee_videos are available' do
    before { make_the_call }

    it '@context.status should be ok' do
      expect(@context.status).to eq 'ok'
    end

    it 'should return pagination meta_data' do
      expect(@context.meta.page).to be 1
      expect(@context.meta.per_page).to be 10
      expect(@context.meta.total_pages).to be 0
      expect(@context.meta.total_results).to be 0
    end

    it 'should return 0 results' do
      expect(@context.videos.length).to be 0
    end
  end

  context 'when no filter params are used' do
    before do
      20.times { |index| create('v1/bee_video', submission_id: index) }

      make_the_call
    end

    it '@context.status should be ok' do
      expect(@context.status).to eq 'ok'
    end

    it 'should return pagination meta_data' do
      expect(@context.meta.page).to be 1
      expect(@context.meta.per_page).to be 10
      expect(@context.meta.total_pages).to be 2
      expect(@context.meta.total_results).to be 20
    end

    it 'should return 10 results' do
      expect(@context.videos.length).to be 10
    end
  end

  context 'when filtering by page = 0' do
    before do
      20.times { create('v1/bee_video') }

      make_the_call page: 0
    end

    it '@context.status should be ok' do
      expect(@context.status).to eq 'ok'
    end

    it 'should return pagination meta_data' do
      expect(@context.meta.page).to be 0
      expect(@context.meta.per_page).to be 0
      expect(@context.meta.total_pages).to be 0
      expect(@context.meta.total_results).to be 20
    end

    it 'should return 20 results' do
      expect(@context.videos.length).to be 20
    end
  end

  context 'when filtering by something' do
    def create_bee_video(trait, options = { user_id: 1 })
      @submission_id += 1

      options[:submission_id] = @submission_id

      create('v1/bee_video', trait, options)
    end

    before do
      @submission_id = 1

      15.times { create_bee_video(:female_laundry) }
      10.times { create_bee_video(:male_hhc) }
      5.times { create_bee_video(:female_hhc) }
      20.times { create_bee_video(:female_hhc, user_id: 2) }

      make_the_call \
        segment_uid: 'hhc',
        filter_query: {
          user_id_eq: 1,
          gender_eq: 'Female'
        }
    end

    it '@context.status should be ok' do
      expect(@context.status).to eq 'ok'
    end

    it 'should return pagination meta_data' do
      expect(@context.meta.page).to be 1
      expect(@context.meta.per_page).to be 10
      expect(@context.meta.total_pages).to be 1
      expect(@context.meta.total_results).to be 5
    end

    it 'should return 5 results' do
      expect(@context.videos.length).to be 5
    end
  end
end
