module Helpers
  def raise_context_unknown_error(context)
    status = context.status

    return if status.ok?
    return if status.created?
    return if status.accepted?
    return if status.not_found?
    return if status.unauthorized?
    return if status.forbidden?
    return if status.unprocessable_entity?
    return if status.bad_request?
    return if status.not_modified?

    raise_hell(context.errors.inspect)
  end

  def raise_hell(message)
    raise DashOverlord::HellRaiserError.new(message)
  end

  def admin_session
    admin_session = {}

    if @current_admin
      admin_session[:admin_id] = @current_admin.id
    end

    admin_session
  end

  def user_session
    user_session = {}

    if @current_user
      user_session[:user_id] = @current_user.id
    end

    user_session
  end

  def app_response(context)
    http_statusses = DashOverlord::Serializers::Base::RAILS_HTTP_STATUS

    OpenStruct.new \
      data: to_hash_recursive(context.data),
      http_status: http_statusses[context.status.to_sym]
  end

  def to_hash_recursive(data)
    if data.is_a? Array
      return data.map { |array_entry| to_hash_recursive(array_entry) }
    end

    if data.respond_to?(:each)
      new_hash = {}

      data.each { |key, value| new_hash[key.to_s] = to_hash_recursive(value) }

      return ActiveSupport::HashWithIndifferentAccess.new new_hash
    end

    data
  end
end
